{
  description = "horizon-spec";
  inputs = {
    flake-utils.url = "github:numtide/flake-utils";
    horizon-platform.url = "git+https://gitlab.homotopic.tech/horizon/horizon-platform";
    lint-utils = {
      url = "git+https://gitlab.homotopic.tech/nix/lint-utils";
      inputs.nixpkgs.follows = "nixpkgs";
    };
    nixpkgs.url = "github:nixos/nixpkgs/nixpkgs-unstable";
  };
  outputs =
    inputs@
    { self
    , flake-utils
    , horizon-platform
    , lint-utils
    , nixpkgs
    , ...
    }:
    flake-utils.lib.eachSystem [ "x86_64-linux" ] (system:
    let
      pkgs = import nixpkgs { inherit system; };

      dhall-prelude = pkgs.dhallPackages.buildDhallGitHubPackage {
        name = "Prelude";
        owner = "dhall-lang";
        repo = "dhall-lang";
        directory = "Prelude";
        file = "package.dhall";
        rev = "v21.1.0";
        sha256 = "sha256-7bIKrdGrlq4NmaFWYWz5Ov2DhslxunoAdUNSMtKGglY=";
        source = true;
      };

      dhall-horizon-spec = pkgs.dhallPackages.buildDhallPackage {
        name = "horizon-spec";
        dependencies = [ dhall-prelude ];
        code = "${./.}/dhall/package.dhall";
      };

      makeTest = (import (nixpkgs + "/nixos/lib/testing-python.nix") { inherit system; }).makeTest;
      hsPkgs = with pkgs.haskell.lib.compose;
        horizon-platform.legacyPackages.${system}.override
          {
            overrides = hfinal: hprev:
              {
                # Setting -f-with-http should disable http, but it doesn't.
                dhall = disableLibraryProfiling (doJailbreak (appendConfigureFlag "-f-with-http" hprev.dhall));
                horizon-spec = dontCheck (disableLibraryProfiling (hprev.callCabal2nix "horizon-spec" ./. { }));
              };
          };
      horizon-spec-tests-wrapper = pkgs.writers.writeBashBin "horizon-spec-tests-wrapper" ''
        DHALL_PRELUDE=${dhall-prelude}/source.dhall ${hsPkgs.horizon-spec}/bin/horizon-spec-tests
      '';
      horizon-spec-tests-check = pkgs.stdenv.mkDerivation {
        name = "horizon-spec-tests-check";
        src = self;
        dontBuild = true;
        installPhase = ''
          export LC_ALL=C.UTF-8
          ${horizon-spec-tests-wrapper}/bin/horizon-spec-tests-wrapper | tee $out
        '';
      };
    in
    {
      devShells.default = hsPkgs.horizon-spec.env.overrideAttrs (attrs: {
        buildInputs = attrs.buildInputs ++ [
          hsPkgs.cabal-install
          pkgs.nixpkgs-fmt
        ];
      });
      packages = {
        default = pkgs.haskell.lib.compose.dontCheck (hsPkgs.horizon-spec);
        dhall-horizon-spec = dhall-horizon-spec;
      };
      checks =
        {
          cabal-fmt = lint-utils.outputs.linters.${system}.cabal-fmt self;
          dhall-format = lint-utils.outputs.linters.${system}.dhall-format self;
          hlint = lint-utils.outputs.linters.${system}.hlint self;
          inherit horizon-spec-tests-check;
          nixpkgs-fmt = lint-utils.outputs.linters.${system}.nixpkgs-fmt self;
          stylish-haskell = lint-utils.outputs.linters.${system}.stylish-haskell self;
        };
    });
}
